/* $Id: TargetProcessDAO.java,v 1.1 2007/11/29 06:01:14 konno Exp $
 *
 * Copyright (c)ARGO 21, Corporation. 2005, 2006.  All rights reserved.
 * 
 * This file is part of Nautica Workflow Core.
 * 
 *  Nautica Workflow Core is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 * 
 *  Nautica Workflow Core is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Nautica Workflow Core; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  
 */
package jp.co.argo21.nautica.workflow.dataaccess;

import java.sql.SQLException;

/**
 * TargetProcessBeanをデータとするDAOインタフェース。
 *
 * @author  nito(Argo 21, Corp.)
 * @version $Revision: 1.1 $
 * @since   Nautica Workflow 1.0
 */
public interface TargetProcessDAO
{
	/**
	 * エンジンIDと起動先プロセスIDで起動先プロセス情報を取得する。
	 *
	 * @param eid エンジンID
	 * @param pid プロセスID
	 * @return 起動先プロセス情報
	 * @throws SQLException 任意のSQL例外
	 * @throws DAOException DAO実行中に処理としてあげる場合がある例外
	 */
	public TargetProcessBean findByPrimaryKey(String eid, String pid)
	throws SQLException, DAOException;
	
	/**
	 * 起動先プロセスIDで起動先プロセス情報を取得する。
	 *
	 * @param pid プロセスID
	 * @return 起動先プロセス情報
	 * @throws SQLException 任意のSQL例外
	 * @throws DAOException DAO実行中に処理としてあげる場合がある例外
	 */
	public TargetProcessBean findByProcessID(String pid)
	throws SQLException, DAOException;
	
	/**
	 * 起動元プロセスIDと起動元アクティビティIDで起動先プロセス情報を取得する。
	 *
	 * @param rootpid 起動元プロセスID
	 * @param rootaid 起動元アクティビティID
	 * @return 起動先プロセス情報
	 * @throws SQLException 任意のSQL例外
	 * @throws DAOException DAO実行中に処理としてあげる場合がある例外
	 */
	public TargetProcessBean findByRootID(String rootpid, String rootaid)
	throws SQLException, DAOException;
	
	/**
	 * 起動先プロセス情報を挿入する。
	 *
	 * @param bean 起動先プロセス情報
	 * @throws SQLException 任意のSQL例外
	 * @throws DAOException DAO実行中に処理としてあげる場合がある例外
	 */
	public void insert(TargetProcessBean bean)
	throws SQLException, DAOException;

	/**
	 * 起動先プロセス情報を削除する。
	 *
	 * @param eid エンジンID
	 * @param pid プロセスID
	 * @throws SQLException 任意のSQL例外
	 * @throws DAOException DAO実行中に処理としてあげる場合がある例外
	 */
	public void delete(String eid, String pid)
	throws SQLException, DAOException;
}
