package jp.co.argo21.nautica.workflow.soap;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.xml.ws.RequestWrapper;
import javax.xml.ws.ResponseWrapper;

import jp.co.argo21.nautica.workflow.soap.message.Attribute;
import jp.co.argo21.nautica.workflow.soap.message.Filter;
import jp.co.argo21.nautica.workflow.soap.message.Participant;
import jp.co.argo21.nautica.workflow.soap.message.WorkItem;
import jp.co.argo21.nautica.workflow.soap.message.WorkItemState;
import jp.co.argo21.nautica.workflow.wfmc.AttributeAssignmentFailedException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidAttributeException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidFilterException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSessionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSourceUserException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidStateException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidTargetUserException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidWorkItemException;
import jp.co.argo21.nautica.workflow.wfmc.TransitionNotAllowedException;

@WebService(name="WorkItemHandler",
        targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
public interface WorkItemHandler {
	
	/**
	 * 作業項目一覧の取得を行う。
	 *
	 * @param session セッションID
	 * @param filter 検索用フィルター
	 * @return 作業項目一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getWorkItems")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getWorkItems", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItems")
	@ResponseWrapper(localName = "getWorkItemsResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemsResponse")   
	public WorkItem[] getWorkItems(@WebParam(name = "session") String session, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidFilterException;
	
	/**
	 * 作業項目の取得を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @return 作業項目
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 */
	@WebMethod(operationName = "getWorkItem")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getWorkItem", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItem")
	@ResponseWrapper(localName = "getWorkItemResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemResponse")   
	public WorkItem getWorkItem(@WebParam(name = "session") String session, @WebParam(name = "wid") String wid) throws InvalidSessionException, InvalidWorkItemException;

	/**
	 * 作業項目の完了を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 */
	@WebMethod(operationName = "completeWorkItem")
	@RequestWrapper(localName = "completeWorkItem", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.CompleteWorkItem")
	public void completeWorkItem(@WebParam(name = "session") String session, @WebParam(name = "wid") String wid) throws InvalidSessionException, InvalidWorkItemException;
	
	/**
	 * 現在の作業項目が遷移可能な状態の一覧を返す。
	 * 状態がない場合は、空の配列を返す。
	 * 利用可能なフィルターはStateFilterのみとする。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param filter 検索用フィルター
	 * @return 作業項目状態一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getWorkItemStates")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getWorkItemStates", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemStates")
	@ResponseWrapper(localName = "getWorkItemStatesResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemStatesResponse")   
	public WorkItemState[] getWorkItemStates(@WebParam(name = "session") String session, @WebParam(name = "wid") String wid, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidFilterException, InvalidWorkItemException;
	
	/**
	 * 作業項目状態の変更を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param state 作業項目状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@WebMethod(operationName = "changeWorkItemState")
	@RequestWrapper(localName = "changeWorkItemState", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.ChangeWorkItemState")
	public void changeWorkItemState(@WebParam(name = "session") String session, @WebParam(name = "wid") String wid, @WebParam(name = "state") WorkItemState state) throws InvalidWorkItemException, InvalidStateException, TransitionNotAllowedException, InvalidSessionException;
	
	/**
	 * 作業項目の再割り当てを行う。
	 *
	 * @param session セッションID
	 * @param sourceUser 割り当て前のユーザ
	 * @param targetUser 割り当て後のユーザ
	 * @param wid 作業項目ID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidTargetUserException 割り当て先ユーザが有効でない場合
	 * @throws InvalidSourceUserException 割り当て前ユーザが有効でない場合
	 */
	@WebMethod(operationName = "reassignWorkItem")
	@RequestWrapper(localName = "reassignWorkItem", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.ReassignWorkItem")
	public void reassignWorkItem(@WebParam(name = "session") String session, @WebParam(name = "sourceUser") Participant sourceUser, @WebParam(name = "targetUser") Participant targetUser, @WebParam(name = "wid") String wid) throws InvalidSessionException, InvalidWorkItemException, InvalidSourceUserException, InvalidTargetUserException;
	
	/**
	 * 作業項目属性一覧の取得を行う。
	 * 利用可能なフィルターはAttributeFilterのみとする。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param filter 検索用フィルター
	 * @return 作業項目属性一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getWorkItemAttributes")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getWorkItemAttributes", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemAttributes")
	@ResponseWrapper(localName = "getWorkItemAttributesResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemAttributesResponse")   
	public Attribute[] getWorkItemAttributes(@WebParam(name = "session") String session, @WebParam(name = "wid") String wid, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidWorkItemException, InvalidFilterException;
	
	
	/**
	 * 作業項目属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 * @throws AttributeAssignmentFailedException 属性の割り当てに失敗した場合
	 */
	@WebMethod(operationName = "assignWorkItemAttribute")
	@RequestWrapper(localName = "assignWorkItemAttribute", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.AssignWorkItemAttribute")
	public void assignWorkItemAttribute(@WebParam(name = "session") String session, @WebParam(name = "wid") String wid , @WebParam(name = "attr") Attribute attr) throws InvalidSessionException, InvalidWorkItemException, InvalidAttributeException, AttributeAssignmentFailedException;
	
	/**
	 * 作業項目属性の取得を行う。
	 *
	 * @param session セッションID
	 * @param wid 作業項目ID
	 * @param name 属性名
	 * @return 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidWorkItemException 作業項目IDが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@WebMethod(operationName = "getWorkItemAttributeValue")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getWorkItemAttributeValue", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemAttributeValue")
	@ResponseWrapper(localName = "getWorkItemAttributeValueResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetWorkItemAttributeValueResponse")   
	public Attribute getWorkItemAttributeValue(@WebParam(name = "session") String session, @WebParam(name = "wid") String wid , @WebParam(name = "name") String name) throws InvalidSessionException, InvalidWorkItemException, InvalidAttributeException;
	
}