package jp.co.argo21.nautica.workflow.soap;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.ws.RequestWrapper;

import jp.co.argo21.nautica.workflow.soap.message.ActivityState;
import jp.co.argo21.nautica.workflow.soap.message.Attribute;
import jp.co.argo21.nautica.workflow.soap.message.Filter;
import jp.co.argo21.nautica.workflow.soap.message.ProcessState;
import jp.co.argo21.nautica.workflow.wfmc.InvalidActivityNameException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidAttributeException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidFilterException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidProcessDefinitionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidProcessInstanceException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSessionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidStateException;
import jp.co.argo21.nautica.workflow.wfmc.TransitionNotAllowedException;

@WebService(name="WorkflowAdminHandler",
        targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
public interface WorkflowAdminHandler {

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、状態の変更を行う。
	 * 指定可能なフィルターはStateFilterとする。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @param state 新しいプロセスインスタンス状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@WebMethod(operationName = "changeProcessesState")
	@RequestWrapper(localName = "changeProcessesState", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.ChangeProcessesState")
	public void changeProcessesState(@WebParam(name = "session") String session , @WebParam(name = "pdid") String pdid, @WebParam(name = "filter") Filter[] filter, @WebParam(name = "state") ProcessState state) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException, InvalidStateException, TransitionNotAllowedException;
	
	/**
	 * 指定されたプロセス定義とアクティビティ定義に属する
	 * アクティビティインスタンスをフィルターで絞り込み、状態の変更を行う。
	 * 指定可能なフィルターはStateFilterとする。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param adid アクティビティ定義ID
	 * @param filter 検索用フィルター
	 * @param state 新しいアクティビティインスタンス状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidActivityNameException 指定されたアクティビティ定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@WebMethod(operationName = "changeActivitiesState")
	@RequestWrapper(localName = "changeActivitiesState", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.ChangeActivitiesState")
	public void changeActivitiesState(@WebParam(name = "session") String session, @WebParam(name = "pdid") String pdid , @WebParam(name = "adid") String adid, @WebParam(name = "filter") Filter[] filter, @WebParam(name = "state") ActivityState state) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidActivityNameException, InvalidFilterException, InvalidStateException, TransitionNotAllowedException;

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、停止を行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "terminateProcesses")
	@RequestWrapper(localName = "terminateProcesses", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.TerminateProcesses")
	public void terminateProcesses(@WebParam(name = "session") String session, @WebParam(name = "pdid") String pdid, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException;

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@WebMethod(operationName = "assignProcessesAttribute")
	@RequestWrapper(localName = "assignProcessesAttribute", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.AssignProcessesAttribute")
	public void assignProcessesAttribute(@WebParam(name = "session") String session, @WebParam(name = "pdid") String pdid, @WebParam(name = "filter") Filter[] filter, @WebParam(name = "attr") Attribute attr) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException, InvalidAttributeException;

	/**
	 * 指定されたプロセス定義とアクティビティ定義に属する
	 * アクティビティインスタンスをフィルターで絞り込み、
	 * 属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param adid アクティビティ定義ID
	 * @param filter 検索用フィルター
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidActivityNameException 指定されたアクティビティ定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@WebMethod(operationName = "assignActivitiesAttribute")
	@RequestWrapper(localName = "assignActivitiesAttribute", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.AssignActivitiesAttribute")
	public void assignActivitiesAttribute(@WebParam(name = "session") String session, @WebParam(name = "pdid") String pdid, @WebParam(name = "adid") String adid, @WebParam(name = "filter") Filter[] filter, @WebParam(name = "attr") Attribute attr) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidActivityNameException, InvalidFilterException, InvalidAttributeException;
	
	 /**
		 * 指定されたプロセスインスタンスの中断を行う。
		 *
		 * @param session セッションID
		 * @param pid プロセスインスタンスID
		 * @throws InvalidSessionException 指定されたセッションが無効の場合
		 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
		 * @throws ServiceAccessorException 通信障害等によりワークフローエンジンと接続できない場合
		 */
	@WebMethod(operationName = "abortProcess")
	@RequestWrapper(localName = "abortProcess", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.AbortProcess")
	public void abortProcess(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid) throws InvalidSessionException, InvalidProcessInstanceException;
	

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、中断を行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "abortProcesses")
	@RequestWrapper(localName = "abortProcesses", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.AbortProcesses")
	public void abortProcesses(@WebParam(name = "session") String session, @WebParam(name = "pdid") String pdid , @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException;

}