package jp.co.argo21.nautica.workflow.soap;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.xml.ws.RequestWrapper;
import javax.xml.ws.ResponseWrapper;

import jp.co.argo21.nautica.workflow.soap.message.Activity;
import jp.co.argo21.nautica.workflow.soap.message.ActivityState;
import jp.co.argo21.nautica.workflow.soap.message.Attribute;
import jp.co.argo21.nautica.workflow.soap.message.Filter;
import jp.co.argo21.nautica.workflow.soap.message.Process;
import jp.co.argo21.nautica.workflow.soap.message.ProcessState;
import jp.co.argo21.nautica.workflow.wfmc.AttributeAssignmentFailedException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidActivityInstanceException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidAttributeException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidFilterException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidProcessDefinitionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidProcessInstanceException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSessionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidStateException;
import jp.co.argo21.nautica.workflow.wfmc.TransitionNotAllowedException;

@WebService(name="WorkflowEngineHandler",
        targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
public interface WorkflowEngineHandler {

	/**
	 * プロセスインスタンスの生成を行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param name プロセスインスタンス名称
	 * @return プロセスインスタンスID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 */
	@WebMethod(operationName = "createProcess")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "createProcess", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.CreateProcess")
	@ResponseWrapper(localName = "createProcessResponseWrapper", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.CreateProcessResponseWrapper")
	public String createProcess(@WebParam(name = "session") String session, @WebParam(name = "pdid") String pdid, @WebParam(name = "name") String name) throws InvalidSessionException, InvalidProcessDefinitionException;
	
	/**
	 * プロセスインスタンスの開始を行う。
	 * 開始した場合に、プロセスインスタンスIDを更新するような
	 * 実装も可能である。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @return 更新されたプロセスインスタンスID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@WebMethod(operationName = "startProcess")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "startProcess", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.StartProcess")
	@ResponseWrapper(localName = "startProcessResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.StartProcessResponse")
	public String startProcess(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid) throws InvalidSessionException, InvalidProcessInstanceException, InvalidAttributeException;
	
	/**
	 * プロセスインスタンスの停止を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 */
	@WebMethod(operationName = "terminateProcess")
	@RequestWrapper(localName = "terminateProcess", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.TerminateProcess")
	public void terminateProcess(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid) throws InvalidSessionException, InvalidProcessInstanceException;

	/**
	 * 指定されたプロセスインスタンスが利用可能な状態一覧の取得を行う。
	 * 利用可能なフィルターはStateFilterのみとする。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param filter 検索用フィルター
	 * @return プロセスインスタンス状態一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getProcessStates")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getProcessStates", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessStates")
	@ResponseWrapper(localName = "getProcessStatesResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessStatesResponse")   
	public ProcessState[] getProcessStates(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidProcessInstanceException, InvalidFilterException;
	
	/**
	 * プロセスインスタンス状態の変更を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param state プロセスインスタンス状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@WebMethod(operationName = "changeProcessState")
	@RequestWrapper(localName = "changeProcessState", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.ChangeProcessState")
	public void changeProcessState(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid , @WebParam(name = "state") ProcessState state) throws InvalidSessionException, InvalidProcessInstanceException, InvalidStateException, TransitionNotAllowedException;
	
	/**
	 * プロセスインスタンス属性一覧の取得を行う。
	 * 利用可能なフィルターはAttributeFilterのみとする。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param filter 検索用フィルター
	 * @return 属性一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getProcessAttributes")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getProcessAttributes", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessAttributes")
	@ResponseWrapper(localName = "getProcessAttributesResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessAttributesResponse")   
	public Attribute[] getProcessAttributes(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidProcessInstanceException, InvalidFilterException;
	
	/**
	 * プロセスインスタンス属性の属性値の取得を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param name 属性名
	 * @return 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@WebMethod(operationName = "getProcessAttributeValue")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getProcessAttributeValue", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessAttributeValue")
	@ResponseWrapper(localName = "getProcessAttributeValueResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessAttributeValueResponse")   
	public Attribute getProcessAttributeValue(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "name") String name) throws InvalidSessionException, InvalidAttributeException;
	
	/**
	 * プロセスインスタンス属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 * @throws AttributeAssignmentFailedException 属性の割り当てに失敗した場合
	 */
	@WebMethod(operationName = "assignProcessAttribute")
	@RequestWrapper(localName = "assignProcessAttribute", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.AssignProcessAttribute")
	public void assignProcessAttribute(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "attr") Attribute attr) throws InvalidSessionException, InvalidProcessInstanceException, InvalidAttributeException, AttributeAssignmentFailedException;

	/**
	 * 指定されたアクティビティインスタンスが利用可能な状態一覧の取得を行う。
	 * 利用可能なフィルターはStateFilterのみとする。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param aid アクティビティインスタンスID
	 * @param filter 検索用フィルター
	 * @return アクティビティインスタンス状態一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidActivityInstanceException アクティビティインスタンスIDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getActivityStates")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getActivityStates", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivityStates")
	@ResponseWrapper(localName = "getActivityStatesResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivityStatesResponse")   
	public ActivityState[] getActivityStates(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "aid") String aid, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidProcessInstanceException, InvalidActivityInstanceException, InvalidFilterException;
	
	/**
	 * アクティビティインスタンス状態の変更を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param aid アクティビティインスタンスID
	 * @param state アクティビティインスタンス状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidActivityInstanceException アクティビティインスタンスIDが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@WebMethod(operationName = "changeActivityState")
	@RequestWrapper(localName = "changeActivityState", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.ChangeActivityState")
	public void changeActivityState(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "aid") String aid, @WebParam(name = "state") ActivityState state) throws InvalidSessionException, InvalidProcessInstanceException, InvalidActivityInstanceException, InvalidStateException, TransitionNotAllowedException;
	
	/**
	 * アクティビティインスタンス属性一覧の取得を行う。
	 * 利用可能なフィルターはAttributeFilterのみとする。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param aid アクティビティインスタンスID
	 * @param filter 検索用フィルター
	 * @return アクティビティインスタンス属性一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidActivityInstanceException アクティビティインスタンスIDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getActivityAttributes")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getActivityAttributes", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivityAttributes")
	@ResponseWrapper(localName = "getActivityAttributesResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivityAttributesResponse")   
	public Attribute[] getActivityAttributes(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "aid") String aid, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidProcessInstanceException, InvalidActivityInstanceException, InvalidFilterException;
	
	/**
	 * アクティビティインスタンス属性の属性値の取得を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param aid アクティビティインスタンスID
	 * @param name 属性名
	 * @return 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@WebMethod(operationName = "getActivityAttributeValue")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getActivityAttributeValue", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivityAttributeValue")
	@ResponseWrapper(localName = "getActivityAttributeValueResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivityAttributeValueResponse")   
	public Attribute getActivityAttributeValue(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "aid") String aid, @WebParam(name = "name") String name) throws InvalidSessionException, InvalidAttributeException;

	/**
	 * アクティビティインスタンス属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param aid アクティビティインスタンスID
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidActivityInstanceException アクティビティインスタンスIDが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 * @throws AttributeAssignmentFailedException 属性の割り当てに失敗した場合
	 */
	@WebMethod(operationName = "assignActivityAttribute")
	@RequestWrapper(localName = "assignActivityAttribute", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.AssignActivityAttribute")
	public void assignActivityAttribute(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "aid") String aid, @WebParam(name = "attr") Attribute attr) throws InvalidSessionException, InvalidProcessInstanceException, InvalidActivityInstanceException, InvalidAttributeException, AttributeAssignmentFailedException;
	
	/**
	 * プロセスインスタンス一覧の取得を行う。
	 * 利用可能なフィルターはNameFilterのみとする。
	 *
	 * @param session セッションID
	 * @param filter 検索用フィルター
	 * @return プロセスの一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getProcesses")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getProcesses", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcesses")
	@ResponseWrapper(localName = "getProcessesResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessesResponse")   
	public Process[] getProcesses(@WebParam(name = "session") String session, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidFilterException;
	
	/**
	 * プロセスインスタンスの取得を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @return プロセスインスタンス
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 */
	@WebMethod(operationName = "getProcess")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getProcess", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcess")
	@ResponseWrapper(localName = "getProcessResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetProcessResponse")   
	public Process getProcess(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid) throws InvalidSessionException, InvalidProcessInstanceException;
	
	/**
	 * アクティビティインスタンス一覧の取得を行う。
	 * 利用可能なフィルターはNameFilterのみとする。
	 *
	 * @param session セッションID
	 * @param filter 検索用フィルター
	 * @return アクティビティインスタンス一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getActivities1")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getActivities1", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivities1")
	@ResponseWrapper(localName = "getActivitiesResponse1", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivitiesResponse1")   
	public Activity[] getActivities(@WebParam(name = "session") String session, @WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidFilterException;
	
	/**
	 * アクティビティインスタンス一覧の取得を行う。
	 * 利用可能なフィルターはNameFilterのみとする。
	 * WfMC仕様ではpidはないが、これは誤りであろうと判断し、
	 * pidを引数に追加した。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param filter 検索用フィルター
	 * @return アクティビティインスタンス一覧
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@WebMethod(operationName = "getActivities2")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getActivities2", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivities2")
	@ResponseWrapper(localName = "getActivitiesResponse2", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivitiesResponse2")   
	public Activity[] getActivities(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid ,@WebParam(name = "filter") Filter[] filter) throws InvalidSessionException, InvalidFilterException;
	
	/**
	 * アクティビティインスタンスの取得を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @param aid アクティビティインスタンスID
	 * @return アクティビティインスタンス
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 * @throws InvalidActivityInstanceException アクティビティインスタンスIDが有効でない場合
	 */
	@WebMethod(operationName = "getActivity")
	@WebResult(name = "return", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message")
	@RequestWrapper(localName = "getActivity", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivity")
	@ResponseWrapper(localName = "getActivityResponse", targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap/message", className = "jp.co.argo21.nautica.workflow.soap.GetActivityResponse")   
	public Activity getActivity(@WebParam(name = "session") String session, @WebParam(name = "pid") String pid, @WebParam(name = "aid") String aid) throws InvalidSessionException, InvalidProcessInstanceException, InvalidActivityInstanceException;
	
}